/* -*- mode: c++; c-basic-offset: 4 -*- */

#include <stdlib.h>
#include <stdio.h>
#include <random>
#include <vector>

int main(int argc, char *argv[])
{
    int n = strtol(argv[1], NULL, 10); // number of options
    int v = strtol(argv[2], NULL, 10); // number of voters
    unsigned m = strtoul(argv[3], NULL, 10); // modulo
    int b = strtol(argv[4], NULL, 10); // number of blocks
    int f = strtol(argv[5], NULL, 10); // number of failed blocks
    unsigned s = strtoul(argv[6], NULL, 10); // random seed
    double d = strtod(argv[7], NULL); // standard deviation
    int t = strtol(argv[8], NULL, 10); // test number

    std::mt19937 rnd(s);
    std::vector<int> votes(v);
    std::vector<int> counts(n);
    for (int i = 0; i < v; ++i) {
        votes[i] = rnd() % n;
        ++counts[votes[i]];
    }

    std::normal_distribution<double> nd(double(v) / b, (double(v) / b) * d);

    // generate sizes of blocks
    std::vector<int> bs(b);
    std::vector<int> ff(b);
    int ind = 0;
    int rb = b;
    int rv = v;
    fprintf(stderr, "block sizes:");
    while (rb > 1) {
        int cur = nd(rnd);
        if (cur <= 0) cur = 1;
        fprintf(stderr, " %d", cur);
        bs[ind++] = cur;
        rv -= cur;
        if (rv <= 0) abort();
        --rb;
    }
    bs[ind] = rv;
    fprintf(stderr, " %d\n", rv);

    // generate numbers of broken blocks
    std::vector<int> numbers(b);
    std::iota(numbers.begin(), numbers.end(), 0);
    std::shuffle(numbers.begin(), numbers.end(), rnd);
    fprintf(stderr, "failed blocks:");
    for (int i = 0; i < f; ++i) {
        ff[numbers[i]] = 1;
        fprintf(stderr, " %d", numbers[i]);
    }
    fprintf(stderr, "\n");

    char p1[64], p2[64];
    snprintf(p1, sizeof(p1), "tests/%03d.dat", t);
    snprintf(p2, sizeof(p2), "tests/%03d.ans", t);

    FILE *df = fopen(p1, "w");
    fprintf(df, "%d\n", n);
    unsigned long long prev_sum = 0;
    ind = 0;
    for (int i = 0; i < b; ++i) {
        int bz = bs[i];
        int f = ff[i];
        fprintf(df, "%d", bz);
        fprintf(df, " %llu", prev_sum);
        prev_sum = 0;
        for (int i = 0; i < bz; ++i) {
            int val = votes[ind++];
            prev_sum += (val + 1);
            if (f && val > 0) --val;
            fprintf(df, " %d", val + 1);
        }
        prev_sum %= m;
        fprintf(df, "\n");
    }
    fprintf(df, "0 %llu\n", prev_sum);
    fclose(df);

    FILE *af = fopen(p2, "w");
    for (int c : counts) {
        fprintf(af, "%d\n", c);
    }
    fclose(af);
}
