/* -*- mode: c; c-basic-offset: 4 -*- */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static void __attribute__((unused, noreturn, format(printf, 1, 2)))
die(const char *format, ...)
{
    va_list args;
    char buf[1024];
    va_start(args, format);
    vsnprintf(buf, sizeof(buf), format, args);
    va_end(args);
    fprintf(stderr, "fatal: %s\n", buf);
    exit(1);
}

enum { PARTY_COUNT = 6 };
enum { MAX_VOTE_COUNT = 251 };

unsigned short *
read_block(int fd)
{
    unsigned short z;
    if (read(fd, &z, sizeof(z)) != sizeof(z)) die("read size");
    if (z > MAX_VOTE_COUNT) die("big block");
    unsigned short *blk = malloc((z + 2) * sizeof(blk[0]));
    if (!blk) die("no memory");
    blk[0] = z;
    int r = read(fd, &blk[1], (z + 1) * sizeof(blk[0]));
    if (r != (z + 1) * sizeof(blk[0])) die("read block size");
    return blk;
}

void
process(const char *path, int *presult)
{
    int fd = open(path, O_RDONLY);
    if (fd < 0) die("open failed");

    unsigned short *cur_block = read_block(fd);
    unsigned short *prev_block = NULL;
    if (!cur_block[0]) return;

    do {
        free(prev_block); prev_block = cur_block;
        cur_block = read_block(fd);
        int sum = cur_block[1];
        int count1 = 0;
        int cur_sum = 0;
        for (int i = 0; i < prev_block[0]; ++i) {
            cur_sum += prev_block[i + 2];
        }
        if (cur_sum == sum) {
            for (int i = 0; i < prev_block[0]; ++i) {
                ++presult[prev_block[i + 2] - 1];
            }
        } else {
            for (int i = 0; i < prev_block[0]; ++i) {
                int vote = prev_block[i + 2];
                if (vote > 1) {
                    sum -= vote + 1;
                    ++presult[vote];
                } else {
                    ++count1;
                }
            }
            int votes1 = sum - count1;
            int votes0 = count1 - votes1;
            presult[1] += votes1;
            presult[0] += votes0;
        }
    } while (cur_block[0]);

    free(prev_block);
    free(cur_block);
}

int
main(int argc, char *argv[])
{
    int result[PARTY_COUNT] = {};
    process(argv[1], result);
    for (int i = 0; i < PARTY_COUNT; ++i) {
        printf("%d\n", result[i]);
    }
}

