/* -*- mode: c; c-basic-offset: 4 -*- */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

static void __attribute__((unused, noreturn, format(printf, 1, 2)))
die(const char *format, ...)
{
    va_list args;
    char buf[1024];
    va_start(args, format);
    vsnprintf(buf, sizeof(buf), format, args);
    va_end(args);
    fprintf(stderr, "fatal: %s\n", buf);
    exit(1);
}

int
main(int argc, char *argv[])
{
    if (argc != 2) die("wrong number of args");
    int fd = open(argv[1], O_WRONLY | O_CREAT | O_TRUNC, 0666);
    if (fd < 0) die("open failed");

    unsigned short prev_sum = 0;
    char *line_s = NULL;
    size_t line_z = 0;
    while (1) {
        free(line_s); line_s = 0;
        line_z = 0;
        ssize_t len = getline(&line_s, &line_z, stdin);
        if (len <= 0) break;

        while (len > 0 && isspace((unsigned char) line_s[len - 1])) --len;
        line_s[len] = 0;
        if (!len) continue;

        char *p = line_s;
        int n = 0;
        int flag;
        if (sscanf(p, "%d%n", &flag, &n) != 1) die("failed to parse flag");
        if (flag != 0 && flag != 1) die("invalid flag");
        p += n;

        char *data = p;
        int count = 0, x;
        int cur_sum = 0;
        while (sscanf(p, "%d%n", &x, &n) == 1) {
            if (x < 1 || x > 6) die("invalid vote");
            p += n;
            cur_sum += x;
            ++count;
        }
        if (*p) die("garbage");
        if (count <= 0 || count > 251) die("invalid count");

        int block_size = (count + 2) * sizeof(short);
        unsigned short *block = malloc(block_size);
        if (!block) die("out of mem");
        block[0] = count;
        block[1] = prev_sum; prev_sum = cur_sum;

        p = data;
        for (int i = 0; i < count; ++i) {
            sscanf(p, "%d%n", &x, &n);
            p += n;
            if (flag && x > 1) --x;
            block[i + 2] = x;
        }

        if (write(fd, block, block_size) != block_size) die("bad write");
    }

    unsigned short last_block[] = { 0, prev_sum };
    if (write(fd, last_block, sizeof(last_block)) != sizeof(last_block))
        die("write error");
    close(fd); fd = -1;
}
